@@ -1,6 +1,7 @@
-from quickstats.components.cli_parser import CLIParser
+#from quickstats.components.cli_parser import CLIParser
+#from quickstats.components.np_ranking_plotter import NPRankingPlotter
 from quickstats.components.extended_minimizer import ExtendedMinimizer
 from quickstats.components.extended_model import ExtendedModel
 from quickstats.components.nuisance_parameter_pull import NuisanceParameterPull
 from quickstats.components.roofit_plotter import RooFitPlotter
-from quickstats.components.np_ranking_plotter import NPRankingPlotter
\ No newline at end of file
+from quickstats.components.nuisance_parameter_harmonizer import NuisanceParameterHarmonizer
