@@ -135,7 +135,7 @@ def compile_macros():
     
     
 @click.command(name='harmonize_np')
-@click.option('ws_files', nargs=-1)
+@click.argument('ws_files', nargs=-1)
 @click.option('-r', '--reference', required=True, help='Path to reference json file containing renaming scheme')
 @click.option('-i', '--input_config_path', default=None, help='Path to json file containing input workspace paths')
 @click.option('-b', '--base_path', default='./', help='Base path for input config')
