@@ -15,14 +15,14 @@ from quickstats.components.numerics import is_integer
 from quickstats.utils.root_utils import load_macro
 
 class ExtendedModel(object):
-    def __init__(self, fname:str, ws_name:str="combWS",
-                 model_config_name:str="ModelConfig", data_name:str="combData", snapshot_name:str="nominalNuis",
+    def __init__(self, fname:str, ws_name:Optional[str]=None, model_config_name:Optional[str]=None,
+                 data_name:str="combData", snapshot_name:Optional[str]="nominalNuis",
                  binned_likelihood:bool=True, tag_as_measurement:str="pdf_",
                  fix_cache:bool=True, fix_multi:bool=True, interpolation_code:int=-1,
                  load_extension:bool=True):
         self.fname = fname
         self.ws_name = ws_name
-        self.model_config_name = model_config_name
+        self.mc_name = model_config_name
         self.data_name = data_name
         self.snapshot_name = snapshot_name
         self.binned_likelihood = binned_likelihood
@@ -112,25 +112,6 @@ class ExtendedModel(object):
             if (component.InheritsFrom(ROOT.RooAbsPdf.Class()) and condition(name)):
                 component.setAttribute("NOCacheAndTrack")
                 print('INFO: Deactivated level 2 constant term optimizatio for {}'.format(name))
-    
-    @staticmethod
-    def get_nuisance_parameter_names(fname:str, ws_name:str, model_config_name:str):
-        ExtendedModel.load_extension()
-        if not os.path.exists(fname):
-            raise FileNotFoundError('workspace file {} does not exist'.format(fname))
-        file = ROOT.TFile(fname) 
-        if (not file):
-            raise RuntimeError("Something went wrong while loading the root file: {}".format(fname))
-        ws = file.Get(ws_name)
-        if not ws:
-            raise RuntimeError("Something went wrong while loading the workspace: {}".format(ws_name))
-        model_config = ws.obj(model_config_name)
-        if not model_config:
-            raise RuntimeError('ERROR: Failed to load model config {}'.format(model_config_name))
-        nuisance_parameters = model_config.GetNuisanceParameters()
-        if not nuisance_parameters:
-            raise RuntimeError('ERROR: Failed to load nuisance parameters')
-        return [nuis.GetName() for nuis in nuisance_parameters]
             
     def initialize(self):
         if not os.path.exists(self.fname):
@@ -139,11 +120,35 @@ class ExtendedModel(object):
         file = ROOT.TFile(self.fname) 
         if (not file):
             raise RuntimeError("Something went wrong while loading the root file: {}".format(self.fname))
-        print('Loading workspace: {}'.format(self.ws_name))
+        # load workspace
+        if self.ws_name is None:
+            ws_names = [i.GetName() for i in file.GetListOfKeys() if i.GetClassName() == 'RooWorkspace']
+            if not ws_names:
+                raise RuntimeError("No workspaces found in the root file: {}".fomat(fname))
+            if len(ws_names) > 1:
+                print("WARNING: Found multiple workspace instances from the root file: {}. Available workspaces"
+                      " are \"{}\". Will choose the first one by default".format(fname, ','.join(ws_names)))
+            self.ws_name = ws_names[0]
+
         ws = file.Get(self.ws_name)
         if not ws:
-            raise RuntimeError("Something went wrong while loading the workspace: {}".format(self.ws_name))
-        
+            raise RuntimeError('failed to load workspace "{}"'.format(self.ws_name))
+        print('INFO: Loaded workspace "{}"'.format(self.ws_name))
+        # load model config
+        if self.mc_name is None:
+            mc_names = [i.GetName() for i in ws.allGenericObjects() if 'ModelConfig' in i.ClassName()]
+            if not mc_names:
+                raise RuntimeError("no ModelConfig object found in the workspace: {}".fomat(ws_name))
+            if len(mc_names) > 1:
+                print("WARNING: Found multiple ModelConfig instances from the workspace: {}. "
+                      "Available ModelConfigs are \"{}\". "
+                      "Will choose the first one by default".format(ws_name, ','.join(mc_names)))
+            self.mc_name = mc_names[0]
+        model_config = ws.obj(self.mc_name)
+        if not model_config:
+            raise RuntimeError('failed to load model config "{}"'.format(self.mc_name))
+        print('INFO: Loaded model config "{}"'.format(self.mc_name))
+            
         # modify interpolation code
         if self.interpolation_code != -1:
             self.modify_interp_codes(ws, self.interpolation_code,
@@ -160,12 +165,6 @@ class ExtendedModel(object):
         # deactivate level 2 constant term optimization
             self.deactivate_lv2_const_optimization(ws, 
                 condition=lambda name: (name.endswith('_mm') and 'mumu_atlas' in name))
-                          
-        # load model config
-        model_config = ws.obj(self.model_config_name)
-        if not model_config:
-            raise RuntimeError('ERROR: Failed to load model config {}'.format(self.model_config_name))
-        print('INFO: Loaded model config "{}" from workspace'.format(self.model_config_name))
 
         # load pdf
         pdf = model_config.GetPdf()
@@ -176,31 +175,31 @@ class ExtendedModel(object):
         # load dataset
         data = ws.data(self.data_name)
         if not data:
-            raise RuntimeError('ERROR: Failed to load dataset')
+            raise RuntimeError('Failed to load dataset')
         print('INFO: Loaded dataset "{}" from workspace'.format(data.GetName()))
                 
         # load nuisance parameters
         nuisance_parameters = model_config.GetNuisanceParameters()
         if not nuisance_parameters:
-            raise RuntimeError('ERROR: Failed to load nuisance parameters')
+            raise RuntimeError('Failed to load nuisance parameters')
         print('INFO: Loaded nuisance parameters from model config')
                 
         # Load global observables
         global_observables = model_config.GetGlobalObservables()
         if not global_observables:
-            raise RuntimeError('ERROR: Failed to load global observables')          
+            raise RuntimeError('Failed to load global observables')          
         print('INFO: Loaded global observables from model config')                  
     
         # Load POIs
         pois = model_config.GetParametersOfInterest()
         if not pois:
-            raise RuntimeError('ERROR: Failed to load parameters of interest')
+            raise RuntimeError('Failed to load parameters of interest')
         print('INFO: Loaded parameters of interest from model config')
                                   
         # Load observables
         observables = model_config.GetObservables()
         if not observables:
-            raise RuntimeError('ERROR: Failed to load observables')     
+            raise RuntimeError('Failed to load observables')     
         print('INFO: Loaded observables from model config')
         
         self._file                = file
@@ -351,7 +350,7 @@ class ExtendedModel(object):
         # if param_expr is None, all parameters will be parsed as None by default
             param_dict = {param.GetName():None for param in source}
         return param_dict
-                          
+
     @staticmethod
     def parse_param_str(param_str):
         '''
@@ -492,70 +491,64 @@ class ExtendedModel(object):
         data = [{'Name':i.GetName(), 'Value':i.getVal(), "Constant":i.isConstant(), "Min":i.getMin(), "Max":i.getMax()} for i in args]
         df = pandas.DataFrame(data)
         return df
-
+    
     @staticmethod
-    def get_workspace(fname:str, ws_name:str="combWS"):
+    def load_ws(fname:str, ws_name:Optional[str]=None, mc_name:Optional[str]=None):
         if not os.path.exists(fname):
             raise FileNotFoundError('workspace file {} does not exist'.format(fname))
-        file = ROOT.TFile(fname) 
+        file = ROOT.TFile(fname)
         if (not file):
-            raise RuntimeError("Something went wrong while loading the root file: {}".format(fname))
+            raise RuntimeError("Something went wrong while loading the root file: {}".format(fname))        
+        # load workspace
+        if ws_name is None:
+            ws_names = [i.GetName() for i in file.GetListOfKeys() if i.GetClassName() == 'RooWorkspace']
+            if not ws_names:
+                raise RuntimeError("No workspaces found in the root file: {}".fomat(fname))
+            if len(ws_names) > 1:
+                print("WARNING: Found multiple workspace instances from the root file: {}. Available workspaces"
+                      " are \"{}\". Will choose the first one by default".format(fname, ','.join(ws_names)))
+            ws_name = ws_names[0]
         ws = file.Get(ws_name)
         if not ws:
-            raise RuntimeError("Something went wrong while loading the workspace: {}".format(ws_name))
-        return ws
-
+            raise RuntimeError('Failed to load workspace: "{}"'.format(ws_name))
+        # load model config
+        if mc_name is None:
+            mc_names = [i.GetName() for i in ws.allGenericObjects() if 'ModelConfig' in i.ClassName()]
+            if not mc_names:
+                raise RuntimeError("no ModelConfig object found in the workspace: {}".fomat(ws_name))
+            if len(mc_names) > 1:
+                print("WARNING: Found multiple ModelConfig instances from the workspace: {}. "
+                      "Available ModelConfigs are \"{}\". "
+                      "Will choose the first one by default".format(ws_name, ','.join(mc_names)))
+            mc_name = mc_names[0]     
+        mc = ws.obj(mc_name)
+        if not mc:
+            raise RuntimeError('Failed to load model config "{}"'.format(mc_name))
+        return file, ws, mc
+    
     @staticmethod
-    def get_model_config(fname:str, ws_name:str="combWS",  model_config_name:str="ModelConfig"):
-        ws = ExtendedModel.get_workspace(fname, ws_name)
-        model_config = ws.obj(model_config_name)
-        if not model_config:
-            raise RuntimeError('ERROR: Failed to load model config {}'.format(model_config_name))
-        return model_config
-
-    @staticmethod       
-    def get_nuisance_parameters(fname:str, ws_name:str="combWS", model_config_name:str="ModelConfig", as_str=True):
-        model_config = ExtendedModel.get_model_config(fname, ws_name, model_config_name)
-        nuisance_parameters = model_config.GetNuisanceParameters()
-        if not nuisance_parameters:
-            raise RuntimeError('ERROR: Failed to load nuisance parameters')
-        if as_str:
-            return [nuis.GetName() for nuis in nuisance_parameters]                
-        else:
-            return nuisance_parameters
-
-    @staticmethod       
-    def get_global_observables(fname:str, ws_name:str="combWS", model_config_name:str="ModelConfig", as_str=True):
-        model_config = ExtendedModel.get_model_config(fname, ws_name, model_config_name)
-        global_observables = model_config.GetGlobalObservables()
-        if as_str:
-            return [go.GetName() for go in global_observables]                
-        else:
-            return global_observables
-
-    @staticmethod       
-    def get_parameters_of_interest(fname:str, ws_name:str="combWS", model_config_name:str="ModelConfig", as_str=True):
+    def get_nuisance_parameter_names(fname:str, ws_name:Optional[str]=None, mc_name:Optional[str]=None):
         ExtendedModel.load_extension()
-        if not os.path.exists(fname):
-            raise FileNotFoundError('workspace file {} does not exist'.format(fname))
-        file = ROOT.TFile(fname) 
-        if (not file):
-            raise RuntimeError("Something went wrong while loading the root file: {}".format(fname))
-        ws = file.Get(ws_name)
-        if not ws:
-            raise RuntimeError("Something went wrong while loading the workspace: {}".format(ws_name))        
-        model_config = ws.obj(model_config_name)
-        pois = model_config.GetParametersOfInterest()
-        if as_str:
-            return [poi.GetName() for poi in pois]                
-        else:
-            return pois            
+        file, ws, mc = ExtendedModel.load_ws(fname, ws_name, mc_name)
+        nuisance_parameters = mc.GetNuisanceParameters()
+        if not nuisance_parameters:
+            raise RuntimeError('Failed to load nuisance parameters')
+        return [nuis.GetName() for nuis in nuisance_parameters]
+    
+    @staticmethod
+    def get_poi_names(fname:str, ws_name:Optional[str]=None, mc_name:Optional[str]=None):
+        ExtendedModel.load_extension()
+        file, ws, mc = ExtendedModel.load_ws(fname, ws_name, mc_name)
+        pois = mc.GetParametersOfInterest()
+        if not pois:
+            raise RuntimeError('Failed to load parameters of interest')        
+        return [poi.GetName() for poi in pois]
 
     @staticmethod
-    def get_datasets(fname:str, ws_name:str="combWS", as_str=True):
-        
-        ws = ExtendedModel.get_workspace(fname, ws_name)
-        if as_str:
-            return [data.GetName() for data in ws.allData()]                
-        else:
-            return [data for data in ws.allData()]  
\ No newline at end of file
+    def get_dataset_names(fname:str, ws_name:Optional[str]=None, mc_name:Optional[str]=None):
+        ExtendedModel.load_extension()
+        file, ws, mc = ExtendedModel.load_ws(fname, ws_name, mc_name)
+        datasets = ws.allData()
+        if not datasets:
+            raise RuntimeError('Failed to load datasets')
+        return [dataset.GetName() for dataset in datasets]
\ No newline at end of file
