@@ -2,6 +2,7 @@ import os
 import time
 import json
 from itertools import repeat
+from typing import List, Optional
 
 import numpy as np
 import ROOT
@@ -18,7 +19,7 @@ def load_cache(fname:str):
         return None
 
 def evaluate_nll(input_file:str, poi_val:float, poi:str="", vmin:float=10., vmax:float=10., unconditional=False,
-                 workspace:str='combWS', model_config:str='ModelConfig', data:str='combData', snapshot:str=None, 
+                 workspace:Optional[str]=None, model_config:Optional[str]=None, data:str='combData', snapshot:str=None, 
                  profile:str="", fix:str="", hesse:bool=False, minos:bool=True, constrain:bool=True, 
                  minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', num_cpu:int=1, binned:bool=True,
                  eps:float=1.0, strategy:int=0, fix_cache:bool=True, fix_multi:bool=True, mpsplit:int=3, 
@@ -35,7 +36,8 @@ def evaluate_nll(input_file:str, poi_val:float, poi:str="", vmin:float=10., vmax
                 print('INFO: Cached NLL for POI "{}" at {:.2f}: {}'.format(poi_name, poi_val, nll))
             return nll
     start = time.time()
-    model = ExtendedModel(fname=input_file, data_name=data, binned_likelihood=binned, snapshot_name=snapshot, 
+    model = ExtendedModel(fname=input_file, ws_name=workspace, model_config_name=model_config,
+                          data_name=data, binned_likelihood=binned, snapshot_name=snapshot, 
                           fix_cache=fix_cache, fix_multi=fix_multi)
     if fix:
         model.fix_parameters(fix)
@@ -107,7 +109,7 @@ def evaluate_nll(input_file:str, poi_val:float, poi:str="", vmin:float=10., vmax
 
 def scan_nll(input_file:str, poi_min:float, poi_max:float, poi_step:float, poi:str="", cache:bool=True,
              outname:str="{poi}", outdir:str='output', vmin:float=10., vmax:float=10., 
-             workspace:str='combWS', model_config:str='ModelConfig', data:str='combData', snapshot:str=None, 
+             workspace:Optional[str]=None, model_config:Optional[str]=None, data:str='combData', snapshot:str=None, 
              profile:str="", fix:str="", hesse:bool=False, minos:bool=True, constrain:bool=True,
              minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', num_cpu:int=1, binned:bool=True,
              eps:float=1.0, strategy:int=0, fix_cache:bool=True, fix_multi:bool=True, mpsplit:int=3, 
@@ -117,7 +119,7 @@ def scan_nll(input_file:str, poi_min:float, poi_max:float, poi_step:float, poi:s
     unconditional = np.concatenate(([True], np.full((len(points)), False)))
     points        = np.concatenate(([0.], points))
     if not poi:
-        poi = ExtendedModel.get_parameters_of_interest(input_file, workspace, model_config)[0]
+        poi = ExtendedModel.get_poi_names(input_file, workspace, model_config)[0]
         print('INFO: POI not given, default as "{}".'.format(poi))
     # try to get the number of significant digits of poi_step
     sd = approx_n_digit(poi_step)
